import os

from testlib.test_base import TestBase, TicEnvironment
from tic.shared.utils.db_util import build_postgresql_connection_string
from tic.shared.file_acquisition.repository.status_update_db import (
    StatusUpdateDbRepository,
)


tic_env = TicEnvironment.get_enum(value=os.environ.get("TEST_ENV", "local"))


class TestStatusUpdateDbBase(TestBase):
    @classmethod
    def setUpClass(cls):
        super(TestStatusUpdateDbBase, cls).setUpClass()
        cls.files_to_clean_up = set()
        cls.aws_region = "us-east-1"
        cls.kms_key_alias = "alias/tic_master"

        # Default settings that assume not local
        db_port = 18018
        db_host = "localhost"
        db_name = cls.base_config["TICBaseConfig"]["status_update_db"][
            "status_update_db_name"
        ]
        db_username = cls.base_config["TICBaseConfig"]["status_update_db"][
            "status_update_db_user"
        ]
        db_password = cls.base_config["TICBaseConfig"]["status_update_db"][
            "status_update_db_pw"
        ]
        db_password_encrypted = cls.base_config["TICBaseConfig"]["status_update_db"][
            "status_update_db_pw_encrypted"
        ]
        db_aws_region = cls.base_config["TICBaseConfig"]["AWS"]["aws_region"]
        cls.db_connection_args = {}

        # Additional config, especially for local usage
        if tic_env == TicEnvironment.ci:
            db_port = 8888
        elif tic_env == TicEnvironment.dev:
            db_port = 18019
        elif tic_env == TicEnvironment.staging:
            db_port = 18020
            cls.test_environment = "staging"

        cls.connection_string = build_postgresql_connection_string(
            host=db_host,
            port=db_port,
            db=db_name,
            username=db_username,
            password=db_password,
            decrypt=db_password_encrypted,
            aws_region=db_aws_region,
        )
        cls.main_repository = StatusUpdateDbRepository(
            None,
            cls.connection_string,
            cls.connection_string,
            connection_args=cls.db_connection_args,
            statement_timeout=0,
        )
