from sqlalchemy.orm import Session
from sqlalchemy.event import listens_for, remove

from testlib.test_status_update_db_base import TestStatusUpdateDbBase


def status_update_db_rollback():
    def decorator(fn):
        def decorated(self, *args, **kwargs):
            if not isinstance(self, TestStatusUpdateDbBase):
                raise TypeError("Test class must be instance of TestStatusUpdateDbBase")
            # Rollback main repo
            rollback(self.main_repository.writer_engine, fn, self, *args, **kwargs)

        return decorated

    return decorator


def rollback(engine, fn, self, *args, **kwargs):
    """
    ref: https://stackoverflow.com/a/36509121
    """
    conn = engine.connect()
    trans = conn.begin()
    session = Session(engine)
    session.begin_nested()

    @listens_for(session, "after_transaction_end")
    def restart_savepoint(_session, transaction):
        if transaction.nested and not transaction._parent.nested:
            _session.begin_nested()

    try:
        fn(self, session, *args, **kwargs)
    finally:
        remove(session, "after_transaction_end", restart_savepoint)
        session.rollback()
        session.close()
        trans.rollback()
        trans.close()
        conn.invalidate()
